/*
 * Copyright (c) 2015, The University of Oxford
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Oxford nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <gtest/gtest.h>
#include <iostream>
#include <climits>
#include "settings/oskar_settings_types.h"

using namespace oskar;

TEST(settings_types, DoubleRange)
{
    DoubleRange r;
    {
        ASSERT_TRUE(r.init("MIN,MAX"));
        ASSERT_TRUE(r.set_default("0.999999999999"));
        ASSERT_TRUE(r.is_default());
        ASSERT_TRUE(r.set_value("2.3"));
        // FIXME(BM) better double to string conversion.
        EXPECT_STREQ("2.3", r.get_value());
        ASSERT_DOUBLE_EQ(2.3, r.value());
        ASSERT_DOUBLE_EQ(0.999999999999, r.default_value());
        ASSERT_DOUBLE_EQ(-DBL_MAX, r.min());
        ASSERT_DOUBLE_EQ(DBL_MAX, r.max());
    }
    {
        ASSERT_TRUE(r.init("5.4, 10.0"));
        ASSERT_DOUBLE_EQ(5.4, r.min());
        ASSERT_DOUBLE_EQ(10.0, r.max());
        ASSERT_FALSE(r.set_value("3.5"));
        ASSERT_DOUBLE_EQ(5.4, r.value());
        ASSERT_FALSE(r.set_value("311.5"));
        ASSERT_DOUBLE_EQ(10.0, r.value());
        ASSERT_TRUE(r.set_value("7.5"));
        ASSERT_DOUBLE_EQ(7.5, r.value());
        ASSERT_FALSE(r.is_default());
    }
}

