/*
 * Copyright (c) 2012-2019, The University of Oxford
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Oxford nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef OSKAR_PRIVATE_STATION_WORK_H_
#define OSKAR_PRIVATE_STATION_WORK_H_

#include <mem/oskar_mem.h>

struct oskar_StationWork
{
    oskar_Mem* horizon_mask;     /* Integer. */
    oskar_Mem* source_indices;   /* Integer. */

    oskar_Mem* enu_direction_x;  /* Real scalar. ENU direction cosine. */
    oskar_Mem* enu_direction_y;  /* Real scalar. ENU direction cosine. */
    oskar_Mem* enu_direction_z;  /* Real scalar. ENU direction cosine. */

    oskar_Mem* theta_modified;   /* Real scalar. */
    oskar_Mem* phi_x;            /* Real scalar. */
    oskar_Mem* phi_y;            /* Real scalar. */
    oskar_Mem* weights;          /* Complex scalar. */
    oskar_Mem* weights_error;    /* Complex scalar. */
    oskar_Mem* array_pattern;    /* Complex scalar. */
    oskar_Mem* beam_out_scratch; /* Output scratch array. */

    /* TEC screen. */
    char screen_type;
    int previous_time_index;
    int screen_num_pixels_x, screen_num_pixels_y, screen_num_pixels_t;
    double screen_height_km;
    double screen_pixel_size_m;
    double screen_time_interval_sec;
    oskar_Mem *tec_screen_path, *tec_screen;
    oskar_Mem *screen_output;

    int num_depths;
    oskar_Mem** beam;            /* For hierarchical stations. */
};

#ifndef OSKAR_STATION_WORK_TYPEDEF_
#define OSKAR_STATION_WORK_TYPEDEF_
typedef struct oskar_StationWork oskar_StationWork;
#endif /* OSKAR_STATION_WORK_TYPEDEF_ */

#endif /* include guard */
